"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const imperative_1 = require("@zowe/imperative");
const zos_jobs_for_zowe_sdk_1 = require("@zowe/zos-jobs-for-zowe-sdk");
const ZosFilesBase_handler_1 = require("../../ZosFilesBase.handler");
/**
 * Handler to compare spooldd's content
 * @export
 */
class LocalfileSpoolddHandler extends ZosFilesBase_handler_1.ZosFilesBaseHandler {
    processWithSession(commandParameters, session) {
        return __awaiter(this, void 0, void 0, function* () {
            const descriptionSeperator = ":";
            const task = {
                percentComplete: 0,
                statusMessage: "Retrieving local file",
                stageName: imperative_1.TaskStage.IN_PROGRESS
            };
            // CHECKING IF LOCAL FILE EXISTS, THEN RETRIEVING IT
            commandParameters.response.progress.startBar({ task });
            let localFile;
            if (path.isAbsolute(commandParameters.arguments.localFilePath)) {
                // resolving to full path if local path passed is not absolute
                localFile = commandParameters.arguments.localFilePath;
            }
            else {
                localFile = path.resolve(commandParameters.arguments.localFilePath);
            }
            const localFileHandle = fs.openSync(localFile, 'r');
            let lfContentBuf;
            try {
                // check if the path given is of a file or not
                try {
                    if (!fs.fstatSync(localFileHandle).isFile()) {
                        throw new imperative_1.ImperativeError({
                            msg: 'Path given is not of a file, do recheck your path again'
                        });
                    }
                }
                catch (error) {
                    if (error instanceof imperative_1.ImperativeError)
                        throw error;
                    throw new imperative_1.ImperativeError({
                        msg: 'Path not found. Please check the path and try again'
                    });
                }
                // reading local file as buffer
                lfContentBuf = fs.readFileSync(localFileHandle);
            }
            finally {
                fs.closeSync(localFileHandle);
            }
            commandParameters.response.progress.endBar();
            // RETRIEVING SPOOLDD
            task.statusMessage = "Retrieving spooldd";
            commandParameters.response.progress.startBar({ task });
            const spoolDescription = commandParameters.arguments.spoolDescription;
            const spoolDescArr = spoolDescription.split(descriptionSeperator);
            const jobName = spoolDescArr[0];
            const jobId = spoolDescArr[1];
            const spoolId = Number(spoolDescArr[2]);
            let spoolContentString = yield zos_jobs_for_zowe_sdk_1.GetJobs.getSpoolContentById(session, jobName, jobId, spoolId);
            commandParameters.response.progress.endBar();
            //CHECKING IF NEEDING TO SPLIT CONTENT STRINGS FOR SEQNUM OPTION
            let lfContentString = "";
            const seqnumlen = 8;
            if (commandParameters.arguments.seqnum === false) {
                lfContentString = lfContentBuf.toString().split("\n")
                    .map((line) => line.slice(0, -seqnumlen))
                    .join("\n");
                spoolContentString = spoolContentString.split("\n")
                    .map((line) => line.slice(0, -seqnumlen))
                    .join("\n");
            }
            else {
                lfContentString = lfContentBuf.toString();
            }
            // CHECK TO OPEN UP DIFF IN BROWSER WINDOW
            if (commandParameters.arguments.browserView) {
                yield imperative_1.DiffUtils.openDiffInbrowser(lfContentString, spoolContentString);
                return {
                    success: true,
                    commandResponse: "Launching local-file and spool-dd diffs in browser...",
                    apiResponse: {}
                };
            }
            // RETURNING DIFF
            const jsonDiff = yield imperative_1.DiffUtils.getDiffString(lfContentString, spoolContentString, {
                outputFormat: 'terminal',
                contextLinesArg: commandParameters.arguments.contextLines
            });
            return {
                success: true,
                commandResponse: jsonDiff,
                apiResponse: {}
            };
        });
    }
}
exports.default = LocalfileSpoolddHandler;
//# sourceMappingURL=LocalfileSpooldd.handler.js.map